@@ -1,8 +1,10 @@
 #!/usr/bin/env python
+
 """`jict` provides dictionaries with multiple levels of nested-ness."""
 
-__version__ = '2.4.1'
-from .jict import jict, sqlconnect, evaluate
+__version__ = '2.4.4'
+from .jict import jict, sqlconnect
+from .helpers import evaluate
 from collections import deque
 
 try:
