@@ -3,11 +3,9 @@ from __future__ import print_function
 from __future__ import division
 
 from collections import defaultdict , deque
-import sys, json, yaml, os, random, re ,copy, importlib
+import sys, json, yaml, os, random, re ,copy, importlib, mmap
 from bson.objectid import ObjectId
-
 from time import time
-
 nolibs = []
 
 try:
@@ -19,14 +17,10 @@ try:
 except:
     nolibs.append('mysql-connector')
 
-# if 'tensorflow' in sys.modules:
-#     import tensorflow
-
 class JSONEncoder(json.JSONEncoder):
     def default(self, o):
         if isinstance(o, ObjectId):
             return str(o)
-
         if isinstance(o, deque):
             return list(o)
 
@@ -44,24 +38,21 @@ def sqlconnect(str):
     , str )[0]
 
     cnt = mysql.connector.connect(
-        host=host, database=database, user=user, password=pawd
+        host=host,
+        database=database,
+        user=user,
+        password=pawd
     )
-
     return cnt
 
-def evaluate(foo, itter=1 , threads = 1 ):
-    # if threads > 1:
-    #     for _ in range(threads):
-    #         t = Thread(target=tht)
-
-    t0 = time()
-    for _ in range(itter):
-        foo()
-    print( time() - t0, "seconds wall time" )
-
 def loader(nd):
     nam, ext = os.path.splitext( nd )
 
+    if not os.path.exists( nd ):
+        file = open( nd, "w+" )
+        file.write('{}')
+        file.close()
+
     file = open( nd, "r+" )
     text = file.read()
     file.close()
@@ -95,13 +86,6 @@ def to_jict(prev):
             nd[k] = i
     return nd
 
-def subscribe(file):
-    jct = jict( file )
-    jct.subsc = True
-    jct.storepath = file
-    while True:
-        yield jct
-
 class jict( defaultdict ):
     generator = None
     storepath = None
@@ -113,30 +97,44 @@ class jict( defaultdict ):
 
         if isinstance( nd, str ):
             try:
-                if len(nd) >= 5 and nd[-5:] in [ '.yaml' , '.json' ]:
-                    share, subsc = False ,False
 
-                    if nd[:6] == 'set://':
-                        share = True
+                if len(nd) >= 5 and nd[-5:] in [ '.yaml' , '.json' ]:
+
+                    if (nd[:6] == 'shm//:' or nd[:6] == 'set://') and nd[-5:] == '.json':
                         nd = nd[6:]
-                    elif nd[:6] == 'get://':
-                        subsc = True
-                        nd = nd[6:]
-                    elif nd[:6] == 'sub://':
-                        return subscribe(nd[6:])
+                        dt = to_jict( loader(nd) )
+                        dt.storepath = nd
+
+                        f = open( nd, "r+b" )
+                        dt.file = mmap.mmap( f.fileno() , 0 )
+                        return dt
 
                     if not os.path.exists( nd ):
                         open( nd , 'w+' ).close()
 
                     dt = to_jict( loader(nd) )
-                    if share: dt.share = True
-                    if subsc: dt.subsc = True
                     dt.storepath = nd
+                elif nd[-4:] == '.env' or '.env.example' in nd:
+                    file = open( nd, "r+" )
+                    text = file.read()
+                    file.close()
+                    jct = jict()
+                    for x in text.split('\n'):
+                        if x == '': continue
+                        if '=' in x:
+                            fields = x.split('=')
+                            if len(fields) == 2:
+                                k,v = fields
+                                jct[k] = v
+
+                    jct.storepath = nd
+                    return jct
                 else:
                     dt = to_jict( json.loads( nd ) )
             except Exception as e:
                 print(e)
                 dt = jict()
+
             return dt
 
 
@@ -165,9 +163,11 @@ class jict( defaultdict ):
             for x in self.generator:
                 yield to_jict(x)
 
-    def init(self,key,deft):
+    # creates a default valuef for the key if doesn't has on
+    def init(self,key, deft ):
         if key in self.keys():
             return self[key]
+
         self[key] = deft
         return self[key]
 
@@ -197,7 +197,8 @@ class jict( defaultdict ):
                 self[key] = val
                 return self[key]
 
-        if self[key] == None: self[key] = 0
+        if self[key] == None:
+            self[key] = 0
 
         self[key] = val if val > self[key] else self[key]
         return self[key]
@@ -209,7 +210,8 @@ class jict( defaultdict ):
                 self[key] = val
                 return self[key]
 
-        if self[key] == None: self[key] = 10 ** 15
+        if self[key] == None:
+            self[key] = 10 ** 15
 
         self[key] = val if val < self[key] else self[key]
         return self[key]
@@ -371,38 +373,85 @@ class jict( defaultdict ):
     def __getitem__(self,key):
         jct = super( defaultdict, self ).__getitem__(key)
 
-        if hasattr( self , 'subsc' ):
-            jct = jict( self.storepath  )
-            return jct[key]
+        # stops the infinite get
+        if hasattr( self , 'skig' ):
+            return jct
+
+        if hasattr( self , 'file' ):
+            mm = self.file
+            mm.seek( 0 )
+            text = mm.read(mm.size())
+            data = json.loads( text )
+            dt = jict( data )
+            val = dt[key]
+
+            setattr(self,'skip',True)
+            self[key] = val
+            del self.skip
+            return val
 
         return jct
 
     def __setitem__( self , key , data ):
         super(defaultdict, self).__setitem__( key , data )
 
-        if hasattr( self , 'share' ):
-            self.save()
+        # stops the infinite set
+        if hasattr( self , 'skip' ):
+            return
+
+        if hasattr( self , 'file' ):
+            mm = self.file
+
+            setattr(self,'skig',True)
+            ns = bytes( self.json( 0 ) , 'utf-8')
+            del self.skig
+            mm.seek( 0 )
+            if mm.size() != len( ns ):
+                mm.resize( len( ns ) )
+
+            mm.seek( 0 )
+            mm.write(ns)
 
     def deque(self,key,data,maxlen = 5 ):
-        if not isinstance(self[key],deque):
-            self[key] = deque(maxlen = maxlen)
+        if not isinstance(self[key], deque ):
+            if isinstance(self[key], list ):
+                self[key] = deque(self[key], maxlen = maxlen)
+            else:
+                self[key] = deque(maxlen = maxlen)
 
-        self[key].append(data)
-        if hasattr(self,'share'):
-            self.save()
+        self[key].append( data )
+        if hasattr( self , 'file' ):
+            mm = self.file
+
+            setattr(self,'skig',True)
+            ns = bytes( self.json( 0 ) , 'utf-8')
+            del self.skig
+
+            mm.seek( 0 )
+            if mm.size() != len( ns ):
+                mm.resize( len( ns ) )
+
+            mm.seek( 0 )
+            mm.write(ns)
+
+    def close(self):
+        if hasattr( self , 'file' ):
+            self.file.flush()
+            self.file.close()
 
     def save(self, name = None, tp = None , shm = '' ):
+        if hasattr( self , 'file' ):
+            return
 
-        if name != None:
-
-            valid = ['sql://']
-
-            for x in valid:
-                if len(name) >= len(x) and name[:len(x)] == x:
-                    if name[:len(x)] == 'sql://':
-                        self.sql_store(name[len(x):])
-                        return
-
+        # if name != None:
+        #
+        #     valid = ['sql://']
+        #
+        #     for x in valid:
+        #         if len(name) >= len(x) and name[:len(x)] == x:
+        #             if name[:len(x)] == 'sql://':
+        #                 self.sql_store(name[len(x):])
+        #                 return
 
         self.storepath = name if name != None else self.storepath \
                     if self.storepath != None else 'jict.json'
@@ -412,25 +461,36 @@ class jict( defaultdict ):
         self.storepath = nam + tp
 
         f = open(self.storepath, "w+")
-        f.write( self.yaml() if tp == '.yaml' else self.json() )
+        if tp == '.example' or tp == '.env':
+            txt = ''
+            for x in self.keys():
+                if isinstance( self[x] ,str):
+                    txt += f'{x}={self[x]}\n'
+
+            f.write( txt )
+        elif tp == '.yaml':
+            f.write( self.yaml() )
+        else:
+            f.write( self.json() )
+
         f.close()
 
-    def sql_store(self,db):
-        connection = sqlconnect()
-
-        skip = ['columns']
-
-        for table in self.keys():
-            lines = self[table]
-
-            if table in skip:
-                continue
-
-            for line in lines:
-                print( line )
-
-        cursor.close()
-        connection.close()
-
-        del cursor
-        del connection
+    # def sql_store(self,db):
+    #     connection = sqlconnect()
+    #
+    #     skip = ['columns']
+    #
+    #     for table in self.keys():
+    #         lines = self[table]
+    #
+    #         if table in skip:
+    #             continue
+    #
+    #         for line in lines:
+    #             print( line )
+    #
+    #     cursor.close()
+    #     connection.close()
+    #
+    #     del cursor
+    #     del connection
